@@ -13,24 +13,13 @@
 #include <boost/mpl/placeholders.hpp>
 #include <boost/type_traits/is_same.hpp>
 #include <boost/proto/core.hpp>
+#include <boost/proto/debug.hpp>
 #include <boost/proto/transform/arg.hpp>
 #include <boost/test/unit_test.hpp>
 
 using namespace boost;
 using namespace proto;
 
-template<typename Grammar, typename Expr>
-void assert_matches(Expr const &expr)
-{
-    BOOST_MPL_ASSERT((matches<Expr, Grammar>));
-}
-
-template<typename Grammar, typename Expr>
-void assert_not_matches(Expr const &expr)
-{
-    BOOST_MPL_ASSERT_NOT((matches<Expr, Grammar>));
-}
-
 struct int_convertible
 {
     int_convertible() {}
@@ -119,24 +108,24 @@ void test_matches()
     assert_matches< terminal<int> >( as_child(1) );
     assert_matches< terminal<int> >( as_expr(1) );
 
-    assert_not_matches< terminal<int> >( lit('a') );
-    assert_not_matches< terminal<int> >( as_child('a') );
-    assert_not_matches< terminal<int> >( as_expr('a') );
+    assert_matches_not< terminal<int> >( lit('a') );
+    assert_matches_not< terminal<int> >( as_child('a') );
+    assert_matches_not< terminal<int> >( as_expr('a') );
 
     assert_matches< terminal<convertible_to<int> > >( lit('a') );
     assert_matches< terminal<convertible_to<int> > >( as_child('a') );
     assert_matches< terminal<convertible_to<int> > >( as_expr('a') );
 
-    assert_not_matches< terminal<int> >( lit((int_convertible())) );
-    assert_not_matches< terminal<int> >( as_child((int_convertible())) );
-    assert_not_matches< terminal<int> >( as_expr((int_convertible())) );
+    assert_matches_not< terminal<int> >( lit((int_convertible())) );
+    assert_matches_not< terminal<int> >( as_child((int_convertible())) );
+    assert_matches_not< terminal<int> >( as_expr((int_convertible())) );
 
     assert_matches< terminal<convertible_to<int> > >( lit((int_convertible())) );
     assert_matches< terminal<convertible_to<int> > >( as_child((int_convertible())) );
     assert_matches< terminal<convertible_to<int> > >( as_expr((int_convertible())) );
 
     assert_matches< if_<is_same<_value, int>() > >( lit(1) );
-    assert_not_matches< if_<is_same<_value, int>() > >( lit('a') );
+    assert_matches_not< if_<is_same<_value, int>() > >( lit('a') );
 
     assert_matches<
         and_<
@@ -145,7 +134,7 @@ void test_matches()
         >
     >( lit(1) );
 
-    assert_not_matches<
+    assert_matches_not<
         and_<
             terminal<_>
           , if_<is_same<_value, int>() >
@@ -172,7 +161,7 @@ void test_matches()
     assert_matches< terminal<char [N]> >( as_child("hello") );
     assert_matches< terminal<char [N]> >( as_expr("hello") );
 
-    assert_not_matches< if_<is_same<_value, int>()> >( lit("hello") );
+    assert_matches_not< if_<is_same<_value, int>()> >( lit("hello") );
 
     assert_matches< terminal<std::string> >( lit(std::string("hello")) );
     assert_matches< terminal<std::string> >( as_child(std::string("hello")) );
@@ -182,13 +171,13 @@ void test_matches()
     assert_matches< terminal<std::basic_string<_> > >( as_child(std::string("hello")) );
     assert_matches< terminal<std::basic_string<_> > >( as_expr(std::string("hello")) );
 
-    assert_not_matches< terminal<std::basic_string<_> > >( lit(1) );
-    assert_not_matches< terminal<std::basic_string<_> > >( as_child(1) );
-    assert_not_matches< terminal<std::basic_string<_> > >( as_expr(1) );
+    assert_matches_not< terminal<std::basic_string<_> > >( lit(1) );
+    assert_matches_not< terminal<std::basic_string<_> > >( as_child(1) );
+    assert_matches_not< terminal<std::basic_string<_> > >( as_expr(1) );
 
-    assert_not_matches< terminal<std::basic_string<_,_,_> > >( lit(1) );
-    assert_not_matches< terminal<std::basic_string<_,_,_> > >( as_child(1) );
-    assert_not_matches< terminal<std::basic_string<_,_,_> > >( as_expr(1) );
+    assert_matches_not< terminal<std::basic_string<_,_,_> > >( lit(1) );
+    assert_matches_not< terminal<std::basic_string<_,_,_> > >( as_child(1) );
+    assert_matches_not< terminal<std::basic_string<_,_,_> > >( as_expr(1) );
 
     #if BOOST_WORKAROUND(__HP_aCC, BOOST_TESTED_AT(61700))
     typedef std::string const const_string;
@@ -198,15 +187,15 @@ void test_matches()
     
     assert_matches< terminal<std::basic_string<_> const & > >( lit(const_string("hello")) );
     assert_matches< terminal<std::basic_string<_> const & > >( as_child(const_string("hello")) );
-    assert_not_matches< terminal<std::basic_string<_> const & > >( as_expr(const_string("hello")) );
+    assert_matches_not< terminal<std::basic_string<_> const & > >( as_expr(const_string("hello")) );
 
     assert_matches< terminal< void(&)() > >( lit(a_function) );
     assert_matches< terminal< void(&)() > >( as_child(a_function) );
     assert_matches< terminal< void(&)() > >( as_expr(a_function) );
 
-    assert_not_matches< terminal< void(*)() > >( lit(a_function) );
-    assert_not_matches< terminal< void(*)() > >( as_child(a_function) );
-    assert_not_matches< terminal< void(*)() > >( as_expr(a_function) );
+    assert_matches_not< terminal< void(*)() > >( lit(a_function) );
+    assert_matches_not< terminal< void(*)() > >( as_child(a_function) );
+    assert_matches_not< terminal< void(*)() > >( as_expr(a_function) );
 
     assert_matches< terminal< convertible_to<void(*)()> > >( lit(a_function) );
     assert_matches< terminal< convertible_to<void(*)()> > >( as_child(a_function) );
@@ -218,7 +207,7 @@ void test_matches()
 
     assert_matches< terminal< void(* const &)() > >( lit(&a_function) );
     assert_matches< terminal< void(* const &)() > >( as_child(&a_function) );
-    assert_not_matches< terminal< void(* const &)() > >( as_expr(&a_function) );
+    assert_matches_not< terminal< void(* const &)() > >( as_expr(&a_function) );
 
     assert_matches<
         or_<
@@ -227,7 +216,7 @@ void test_matches()
         >
     >( lit(1) );
 
-    assert_not_matches<
+    assert_matches_not<
         or_<
             if_<is_same<_value, char>() >
           , if_<is_same<_value, int>() >
@@ -235,19 +224,19 @@ void test_matches()
     >( lit(1u) );
 
     assert_matches< Input >( cin_ >> 1 >> 2 >> 3 );
-    assert_not_matches< Output >( cin_ >> 1 >> 2 >> 3 );
+    assert_matches_not< Output >( cin_ >> 1 >> 2 >> 3 );
 
     assert_matches< Output >( cout_ << 1 << 2 << 3 );
-    assert_not_matches< Input >( cout_ << 1 << 2 << 3 );
+    assert_matches_not< Input >( cout_ << 1 << 2 << 3 );
 
     assert_matches< function< terminal<int>, vararg< terminal<char> > > >( lit(1)('a','b','c','d') );
-    assert_not_matches< function< terminal<int>, vararg< terminal<char> > > >( lit(1)('a','b','c',"d") );
+    assert_matches_not< function< terminal<int>, vararg< terminal<char> > > >( lit(1)('a','b','c',"d") );
 
     assert_matches< Anything >( cout_ << 1 << +lit('a') << lit(1)('a','b','c',"d") );
 
     assert_matches< proto::switch_<MyCases> >( lit(1) >> 'a' );
     assert_matches< proto::switch_<MyCases> >( lit(1) + 'a' );
-    assert_not_matches< proto::switch_<MyCases> >( lit(1) << 'a' );
+    assert_matches_not< proto::switch_<MyCases> >( lit(1) << 'a' );
 
     number<int, two_complement_c> num;
     assert_matches<NumberGrammar>(proto::as_expr(num));
@@ -257,11 +246,11 @@ void test_matches()
         proto::nullary_expr<my_terminal, int>::type i = {0};
 
         assert_matches<proto::nullary_expr<my_terminal, _> >( i );
-        assert_not_matches<proto::terminal<_> >( i );
+        assert_matches_not<proto::terminal<_> >( i );
 
         proto::terminal<int>::type j = {0};
         assert_matches<proto::terminal<_> >( j );
-        assert_not_matches<proto::nullary_expr<my_terminal, _> >( j );
+        assert_matches_not<proto::nullary_expr<my_terminal, _> >( j );
 
         assert_matches<proto::nullary_expr<_, _> >( i );
     }
@@ -269,7 +258,7 @@ void test_matches()
     // check 0 and 1 arg forms or or_ and and_
     {
         assert_matches< proto::and_<> >( lit(1) );
-        assert_not_matches< proto::or_<> >( lit(1) );
+        assert_matches_not< proto::or_<> >( lit(1) );
 
         assert_matches< proto::and_<proto::terminal<int> > >( lit(1) );
         assert_matches< proto::or_<proto::terminal<int> > >( lit(1) );
